"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var deep_diff_1 = __importDefault(require("deep-diff"));
// https://github.com/flitbit/diff#differences
var dictionary = {
    E: {
        color: '#2196F3',
        text: 'CHANGED:'
    },
    N: {
        color: '#4CAF50',
        text: 'ADDED:'
    },
    D: {
        color: '#F44336',
        text: 'DELETED:'
    },
    A: {
        color: '#2196F3',
        text: 'ARRAY:'
    }
};
function style(kind) {
    return "color: " + dictionary[kind].color + "; font-weight: bold";
}
function styleSuccess() {
    return "color: mediumseagreen; font-weight: bold";
}
function styleLight() {
    return 'background: gray; font-weight: bold; font-size: 0.8rem; padding: 2px;';
}
function render(diff) {
    var kind = diff.kind, path = diff.path, lhs = diff.lhs, rhs = diff.rhs, index = diff.index, item = diff.item;
    switch (kind) {
        case 'E':
            return [path.join('.'), "" + lhs, '→', "" + rhs];
        case 'N':
            return [path.join('.'), "" + rhs];
        case 'D':
            return [path.join('.')];
        case 'A':
            return [path.join('.') + "[" + index + "]", item];
        default:
            return [];
    }
}
function diffLogger(prevState, newState, topic) {
    if (topic === void 0) { topic = null; }
    var diff = deep_diff_1.default(prevState, newState);
    if (topic)
        console.group("%cState changed: " + topic, styleLight());
    console.log('%c Previous State', style('D'), prevState);
    console.group('%c Update', style('E'));
    if (diff) {
        diff.forEach(function (elem) {
            var kind = elem.kind;
            var output = render(elem);
            console.log.apply(console, ["%c " + dictionary[kind].text, style(kind)].concat(output));
        });
    }
    else {
        console.log('—— None ——');
    }
    console.groupEnd();
    console.log('%c Next State', styleSuccess(), newState);
    if (topic)
        console.groupEnd();
}
exports.default = diffLogger;
